
/* Generated data (by glib-mkenums) */

#ifndef __SUSHI_ENUM_TYPES_H__
#define __SUSHI_ENUM_TYPES_H__

#include <glib-object.h>

G_BEGIN_DECLS
/* enumerations from "libsushi/sushi-sound-player.h" */
GType sushi_sound_player_state_get_type (void) G_GNUC_CONST;
#define SUSHI_TYPE_SOUND_PLAYER_STATE (sushi_sound_player_state_get_type())
G_END_DECLS

#endif /* !__SUSHI_ENUM_TYPES_H__ */

/* Generated data ends here */

